@@ -10,25 +10,23 @@ CELERY_ALWAYS_EAGER = True  # Forces celery to run locally for testing
 
 INSTALLED_APPS += [
     'moons',
-    'corptools'
+    'corptools',
+    'invoices'
 ]
 
 ROOT_URLCONF = 'tests.urls'
 
 NOSE_ARGS = [
-    #'--with-coverage',
-    #'--cover-package=',
-    #'--exe',  # If your tests need this to be found/run, check they py files are not chmodded +x
+    # '--with-coverage',
+    # '--cover-package=',
+    # '--exe',  # If your tests need this to be found/run, check they py files are not chmodded +x
 ]
 
-CACHES['default'] = {'BACKEND': 'django.core.cache.backends.db.DatabaseCache'}
-
-
 PASSWORD_HASHERS = [
     'django.contrib.auth.hashers.MD5PasswordHasher',
 ]
 
-#LOGGING = None  # Comment out to enable logging for debugging
+LOGGING = None  # Comment out to enable logging for debugging
 
 # Register an application at https://developers.eveonline.com for Authentication
 # & API Access and fill out these settings. Be sure to set the callback URL
@@ -39,4 +37,17 @@ ESI_SSO_CLIENT_ID = '123'
 ESI_SSO_CLIENT_SECRET = '123'
 ESI_SSO_CALLBACK_URL = '123'
 
-
+CACHES = {
+    "default": {
+        # "BACKEND": "redis_cache.RedisCache",
+        # "LOCATION": "localhost:6379",
+        # "OPTIONS": {
+        #    "DB": 1,
+        # }
+        "BACKEND": "django_redis.cache.RedisCache",
+        "LOCATION": "redis://localhost:6379/1",
+        "OPTIONS": {
+            "COMPRESSOR": "django_redis.compressors.lzma.LzmaCompressor",
+        }
+    }
+}
